#include <iostream>
#include <iomanip>
#include <sstream>
#include <deque>
#include <cstdlib>

#include "TRandom3.h" 
#include "TObject.h"

#include "NMadoka.hh"
#include "CDataSample.hh"

void OneRun(Int_t runnumber);
void OneEvent(Int_t eventnumber);

static const Int_t RUN = 10;
static const Int_t EVENT = 10000;
static const Int_t INNEREVENT = 100;

int main()
{
  for(Int_t runnumber = 0; runnumber < RUN; ++runnumber){
    OneRun(runnumber);
  }

  return 0;
}

void OneRun(Int_t runnumber)
{
  std::cout << "run:" << runnumber << std::endl;

  CDataSample* datasample = new CDataSample("Nadeko");

  std::ostringstream oss;
  oss << "root/run" << std::setw(4) << std::setfill('0') << runnumber << ".root";
  NMadoka::CreateFile(oss.str().c_str(), "recreate");
  NMadoka::CreateTree("Koyomi", "Koyomi is Hero of Bakemono-Gatari");
  NMadoka::CreateBranch("branchname", &datasample, "Koyomi");

  for(Int_t i = 0; i < EVENT; ++i){
    OneEvent(i);
  }

  std::cout << std::endl;

  NMadoka::Write();
}

void OneEvent(Int_t eventnumber)
{
  std::cout << "\r" << "event:" << eventnumber << std::flush;

  TRandom3* random = new TRandom3;
  random->SetSeed();

  double x,y,z;
  double r = random->Rndm();
  random->Sphere(x,y,z,r);

  //fast but danger
#if 1
  CDataSample* datasample = (CDataSample*)NMadoka::FindObject("Nadeko");
  if(!datasample) std::abort();
#endif

  //slow but safe
#if 0
  CDataSample* datasample = 0;
  TObject* obj = NMadoka::FindObject("Nadeko");
  if(obj && obj->InheritsFrom("CDataSample")){
    datasample = (CDataSample*)obj;
  }else{
    std::abort();
  }
#endif

  datasample->Reset();

  datasample->fEventFlag = random->Integer(2);
  datasample->x = x;
  datasample->y = y;
  datasample->z = z;      
  datasample->r = r;
  for(UInt_t j = 0; j < 1+random->Integer(INNEREVENT); ++j){
    datasample->deque_d.push_back(x);
    datasample->deque_d.push_back(y);
    datasample->deque_d.push_back(z);
  }

  NMadoka::Fill("Koyomi");

  delete random;
}
